# {{ package.pypi_name }}

[![PyPI - {{ package.pypi_name }}]({{ package.url.pypi_badge }})]({{ package.url.pypi }})
[![PyPI - Python Version]({{ package.url.pyversions_badge }})]({{ package.url.pypi }})
[![Docs]({{ package.url.rtd_badge }})]({{ package.url.docs }})
[![PyPI - Downloads]({{ package.url.pepy_badge }})]({{ package.url.pypistats }})

![boto3.typed]({{ builder_repo_url }}/raw/main/logo.png)

Type annotations for
[{{ package.library_name }}{% if service_name %} {{ service_name.class_name }}{% endif %} {{ package.library_version }}]({{ package.url.library_pypi }})
compatible with
[VSCode](https://code.visualstudio.com/),
[PyCharm](https://www.jetbrains.com/pycharm/),
[Emacs](https://www.gnu.org/software/emacs/),
[Sublime Text](https://www.sublimetext.com/),
[mypy](https://github.com/python/mypy),
[pyright](https://github.com/microsoft/pyright)
and other tools.

Generated with [{{ builder_package_name }} {{ builder_version }}]({{ builder_repo_url }}).

{% if package.data.pypi_stubs_name != package.pypi_name %}
More information can be found on
[{{ package.data.pypi_stubs_name }}]({{ package.url.stubs_pypi }}) page
and in [{{ package.pypi_name }} docs]({{ package.get_local_doc_link() }}).
{% else %}
More information can be found in [{{ package.data.pypi_stubs_name }} docs]({{ package.get_local_doc_link() }}).
{% endif %}

See how it helps you find and fix potential bugs:

![types-boto3 demo]({{ builder_repo_url }}/raw/main/demo.gif)
