# Type annotations for {{ package.library_name }}

Auto-generated documentation for [{{ package.library_name }}]({{ package.url.library_pypi }})
type annotations package [{{ package.pypi_name }}]({{ package.url.pypi }}).

Generated with [{{ builder_package_name }} {{ builder_version }}]({{ builder_repo_url }}).

{% include "common/how_to_install.md.jinja2" with context %}

## Packages
- [{{ package.data.pypi_stubs_name }}]({{ package.url.stubs_pypi }}) - [{{ package.library_name }}]({{ package.url.library_pypi }}) type annotations with `session.client/resource` overloads
- [{{ package.data.pypi_lite_name }}]({{ package.url.stubs_lite_pypi }}) - [{{ package.library_name }}]({{ package.url.library_pypi }}) type annotations without `session.client/resource` overloads
- [{{ package.data.pypi_full_name }}]({{ package.url.stubs_full_pypi }}) - type annotations for all services below in one package
{% for service_name in package.service_names %}
- [{{ package.get_service_pypi_name(service_name) }}](./{{ package.get_module_name(service_name) }}/README.md) - type annotations for [{{ service_name.class_name }}]({{ service_name.boto3_doc_link }}) service
{% endfor %}
