"""
Type annotations for {{ package.library_name }}.__init__ module.

{{ copyright }}
"""
import logging
from typing import Any
from boto3 import session as session
from boto3.session import Session as Session
from botocore.session import Session as BotocoreSession

{% for import_record in package.get_init_required_import_records() -%}
    {{ import_record -}}{{ "\n" -}}
{% endfor -%}

__author__: str = ...
__version__: str = ...

DEFAULT_SESSION: Session | None = ...

def setup_default_session(
    *,
    aws_access_key_id: str | None = ...,
    aws_secret_access_key: str | None = ...,
    aws_session_token: str | None = ...,
    region_name: str | None = ...,
    botocore_session: BotocoreSession | None = ...,
    profile_name: str | None = ...,
) -> None: ...
def set_stream_logger(
    name: str = ...,
    level: int = ...,
    format_string: str | None = ...,
) -> None: ...
def _get_default_session() -> Session: ...

class NullHandler(logging.Handler): ...

{% for function in package.init_functions -%}
    {% include "common/function.py.jinja2" with context -%}
    {{ '\n' -}}
{% endfor -%}