"""
Postprocessor for aioboto3 classes and methods.

Copyright 2024 Vlad Emelianov
"""

from mypy_boto3_builder.postprocessors.aiobotocore import AioBotocorePostprocessor


class AioBoto3Postprocessor(AioBotocorePostprocessor):
    """
    Postprocessor for aioboto3 classes and methods.
    """
